﻿Imports SimpleDWriteLib
Imports Windows.Foundation
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Media
Imports System

Namespace RippleText
    Partial Public NotInheritable Class MainPage
        Inherits Page
        Private txtGeoGenerator As New TextGeometryGenerator
        Private originalGeometry As PathGeometry
        Private transformedGeometry As PathGeometry
        Private geometryBounds As Rect

        Public Sub New()
            Me.InitializeComponent()

            txtGeoGenerator.Text = "Ripple Text"
            txtGeoGenerator.FontSize = 192
            txtGeoGenerator.Interpolate = True

            lstbox.ItemsSource = txtGeoGenerator.FontFamilies
            lstbox.SelectedItem = "Times New Roman"

            AddHandler CompositionTarget.Rendering, AddressOf OnCompositionTargetRendering
        End Sub

        Private Sub OnListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
            If lstbox.SelectedItem Is Nothing Then
                originalGeometry = Nothing
                transformedGeometry = Nothing
                path.Data = Nothing
            Else
                txtGeoGenerator.FontFamily = CStr(lstbox.SelectedItem)
                originalGeometry = txtGeoGenerator.FlattenedGeometry
                geometryBounds = originalGeometry.Bounds

                ' Force TextGeometryGenerator to generate a new geometry
                Dim text As String = txtGeoGenerator.Text
                txtGeoGenerator.Text = ""
                txtGeoGenerator.Text = text

                ' Save it and set to to the Path
                transformedGeometry = txtGeoGenerator.FlattenedGeometry
                path.Data = transformedGeometry
            End If
        End Sub

        Private Sub OnCompositionTargetRendering(ByVal sender As Object, ByVal arg As Object)
            If originalGeometry Is Nothing OrElse transformedGeometry Is Nothing Then
                Exit Sub
            End If

            Dim renderingArgs As RenderingEventArgs = TryCast(arg, RenderingEventArgs)
            Dim t As Double = renderingArgs.RenderingTime.TotalSeconds / 10

            Dim figIndex As Integer
            For figIndex = 0 To originalGeometry.Figures.Count - 1
                Dim srcPathFigure As PathFigure = originalGeometry.Figures(figIndex)
                Dim dstPathFigure As PathFigure = transformedGeometry.Figures(figIndex)

                Dim segIndex As Integer
                For segIndex = 0 To srcPathFigure.Segments.Count - 1
                    Dim srcPoints As PointCollection = Nothing
                    Dim dstPoints As PointCollection = Nothing

                    If TypeOf srcPathFigure.Segments(segIndex) Is PolyLineSegment Then
                        srcPoints = (TryCast(srcPathFigure.Segments(segIndex), PolyLineSegment)).Points
                        dstPoints = (TryCast(dstPathFigure.Segments(segIndex), PolyLineSegment)).Points
                    Else
                        srcPoints = (TryCast(srcPathFigure.Segments(segIndex), PolyBezierSegment)).Points
                        dstPoints = (TryCast(dstPathFigure.Segments(segIndex), PolyBezierSegment)).Points
                    End If

                    Dim ptIndex As Integer
                    For ptIndex = 0 To srcPoints.Count - 1
                        Dim srcPt As Point = srcPoints(ptIndex)

                        Dim horz As Double = (srcPt.X - geometryBounds.Left) / geometryBounds.Width
                        Dim vert As Double = (srcPt.Y - geometryBounds.Top) / geometryBounds.Height * 2 - 1

                        ' Alternatively, set vert to 1 (or other constant) for ripple that doesn't change character height
                        ' vert = 1

                        Dim dstPt As New Point(srcPt.X, srcPt.Y + 30 * vert * Math.Sin((t + horz) * 10 * Math.PI))
                        dstPoints(ptIndex) = dstPt
                    Next ptIndex
                Next segIndex
            Next figIndex
        End Sub
    End Class
End Namespace
